import json
from langchain_core.prompts import PromptTemplate
from langchain_openai import ChatOpenAI
from pydantic import BaseModel, Field
from langchain_core.output_parsers import PydanticOutputParser
from ..tasks.baas_clsname_task import BassClsNameTask
from ..tasks.serper_task import SerperTask

class BlogContent(BaseModel):
    summary: str = Field(description="블로그 요약")
    content_step1: str = Field(description="블로그 내용 1단계")
    content_step2: str = Field(description="블로그 내용 2단계")
    content_step3: str = Field(description="블로그 내용 3단계")
    result: str = Field(description="블로그 내용 최종")
    main_image: str = Field(description="블로그 메인 이미지")
    medium_image: str = Field(description="블로그 중간 이미지")
    sub_image1: str = Field(description="블로그 서브 이미지 1")
    sub_image2: str = Field(description="블로그 서브 이미지 2")

class BlogInsight(BaseModel):
    category: str = Field(description="블로그 카테고리")
    title: str = Field(description="블로그 제목")
    writer: str = Field(description="AI 블로거")
    meta_description: str = Field(description="블로그 메타 설명")
    write_date: str = Field(description="2024-12-17")
    content: BlogContent = Field(description="블로그 내용")



class BloggerAgent:
    def __init__(self, openai_key:str):
        self.baas_task = BassClsNameTask("HAND")
        self.llm = ChatOpenAI(model="gpt-4o-mini", api_key=openai_key)

    def run(self, subject:str):
         #  ChatOllama(model="llama3.2-vision") # 
        parser = PydanticOutputParser(pydantic_object=BlogInsight)
        prompt = PromptTemplate.from_template("""
            너는 쉽고 재미있게 글을 작성하는데 뛰어난 재능을 가진 블로거이다.  
            다음의 {subject} 에 대한 블로그를 작성해줘.
            이해하기 쉽고 재미있지만, 중요한 내용은 포함해줘.
            그리고 내용은 반드시 전체 10000자 이상 작성해줘.
            그리고 표현은 했습니다 와 같은 존어체로 작성해줘.
            반드시 모르면 모른다고 답변해줘. 절대로 할루션네이션으로 없는 이야기를 만들면 안돼.                                  
                                              
            그리고 형태는 아래 포맷을 이용해서 동일한 형태로 작성해줘
            FORMAT:
            {format_instructions}
        """)
        prompt = prompt.partial(format_instructions=parser.get_format_instructions())
        chain = prompt | self.llm 
        res = chain.invoke({"subject": subject})
        return res
    

    def get_blog_content(self, subject:str):
        res = self.run(subject)
        content = res.content
        content = content.replace("```json", "")
        content = content.replace("```", "")
        return json.loads(content)

    def make_subject(self, main_subject:str):
        serper_task = SerperTask()
        res = serper_task.run(main_subject)
        
        return res

    def save_blog_content(self, main_subject:str):
        subject = self.make_subject(main_subject)
        if subject is None:
            return "블로그 주제를 찾을 수 없습니다."
        print(f"subject: {subject}")
        res = self.get_blog_content(subject)
        self.baas_task.basicPost(class_name="BlogPosts", data=res)


