import logging
import time
from datetime import datetime, date, timedelta
from time import sleep
import multiprocessing
import threading
from ..tasks.async_background_task import AsyncBackgroundTask
from ..tasks.baas_clsname_task import BassClsNameTask
from ..tasks.ha_common_task import HaCommonTask
from ..model.youtube_info import YoutubeInfo
from ..agents.daglo_agent import DagloAgent
from ..agents.whisper_agent import WhisperAgent
from ..ha_constant import HaConstant
class CronAgent:
    def __init__(self):
        self.logger = logging.getLogger('CronAgent')
        self.task_async_background = AsyncBackgroundTask()
        self.task_baas = BassClsNameTask()
        self.task_common = HaCommonTask()
        self.daglo_agent = DagloAgent()
        self.class_name_master = "YoutubeMaster"
        self.class_name_detail = "YoutubeDetail_"
        self.class_name_daily = "YoutubeMasterDaily"
        self.agent_whisper = WhisperAgent()
        self.embed_dir = HaConstant.embed_dir


    def get_youtube_master_info(self, order_key:str="author"):    
        data = self.task_baas.basicOrderGet(class_name=self.class_name_master, orderKey=order_key)
        try:
            result = data.json()
            return result["results"]
        except Exception as e:
            print(e)
            return []
        
    def check_range_date(self, publish_date:str):
        today = date.today() 
        one_week = timedelta(days=7)
        one_week_ago = today - one_week

        ref_date = datetime.strptime(f"{one_week_ago}", "%Y-%m-%d")
        target_date = datetime.strptime(publish_date, "%Y-%m-%d %H:%M:%S")
        check = ref_date < target_date
        return check    

    def cron_youtube_daily_recent_week(self, check, objectId, author, index, publish_date, updateObjectId):
            if check:
                class_name_current = self.class_name_detail+objectId
                res = self.task_baas.basicOrderGet(class_name=class_name_current, orderKey="-publish_date")
                if res.status_code == 200:  
                    results = res.json()['results']
                    if len(results) > 0:
                        data = results[0]
                        ### 아래 객체에서 제거해야 하는 항목 처리
                        data.pop('objectId',None)
                        data.pop('createdAt',None)
                        data.pop('updatedAt',None)
                        data['cron_date'] = datetime.now().strftime("%Y-%m-%d")

                        exist_check = self.task_baas.exist_key_date(class_name=self.class_name_daily, keyName="objectId", data=objectId)
                        if exist_check:
                            print(f"이미 존재하는 데이터입니다. {objectId}")
                        else:
                            res_save = self.task_baas.basicPost(class_name=self.class_name_daily, data=data)
                            print(f"res_save: {res_save.status_code} || {res_save.content}")
                            if res_save.status_code == 201:
                                print(f"저장 완료 {author} {index+1}")
                                # master 업데이트
                                video_id = self.task_common.get_audio_id(url=data['watch_url'])
                                update_data = {
                                    "title": data['title'],
                                    "thumbnail_url": data['thumbnail_url'],
                                    "watch_url": data['watch_url'],
                                    "publish_date": publish_date,
                                    "description": data['description'],
                                    "video_length": data['video_length'],
                                    "video_id": video_id,
                                }
                                self.task_baas.basicPut(class_name=self.class_name_master, object_id=updateObjectId, data=update_data)
                    
      
                    else:
                        print(results)
                else:
                    print(res.content)


    
            

    def get_yt(self,url:str):
        try:
            yt = self.task_common.get_youtube_info(url)
            return yt
        except Exception as e:
            if "403" in str(e):
            # 403 에러시 토큰 재생성
                self.task_common.regenerate_network_token(url=url)
                sleep(10)
            print(e)
            return None
        
    def get_yt_info(self, url:str):
        yt = self.get_yt(url)
        info = YoutubeInfo()
        data = info.change_model(yt=yt) 
        return data    

    def save_yt_master(self, url:str, data:dict):
        yt = self.get_yt(url=url)
        channel_id = yt.channel_id
        check = self.task_baas.exist_key_date(class_name=self.class_name_master, keyName="channel_id", data=channel_id) 
        if check:
            print(f"{yt.author} [{channel_id}]이미 존재합니다.")
        else:
            res = self.task_baas.basicPost(class_name=self.class_name_master, data=data)
            check = res.status_code == 200
            data["check"] = check
            print(f"{yt.author} [{channel_id}]저장 되었습니다.")
        return data
    

    # 채널내 신규 등록시 RAG 처리되는 함수
    def update_rag_by_new(self, channel_id:str):
        # master 에서 대상채널 조회
        master_res = self.get_master_list()
        master_results = master_res.json()['results']
        watch_url_list = []
        for master_result in master_results:
            oid = master_result['objectId']
            get_channel_id = master_result['channel_id']
            # print(f"[update_rag_by_new] get_channel_id: {get_channel_id} || channel_id: {channel_id}")
            if get_channel_id == channel_id:
                class_name = self.class_name_detail + oid
                res = self.task_baas.get_key_not_equal_to(class_name=class_name, keyName="is_rag", data=True)
                results = res['results']
                print(f"[cron_whisper_not_updated] class_name: {class_name}")
            
                if len(results) > 0:
                    for result in results:
                        watch_url_list.append(result['watch_url'])

        for url in watch_url_list:
            print(f"[업데이트 필요 RAG 작업] url: {url} || {len(watch_url_list)}")
            self.whisper_action(url)
    


    def save_yt_detail(self, video_urls:list, data:dict, limit:int=2):
        list_data = []
        for youtube_url in video_urls:    
            vedios = self.task_common.get_channel_urls(url=youtube_url, limit=limit, sleep=2)
            for index, video in enumerate(vedios):
                sleep(1)

                data = self.get_yt_info(video)
                channel_id = data["channel_id"]
                try:

                    object_id = self.task_baas.get_object_id(class_name=self.class_name_master, keyName="channel_id", data=channel_id)
                    class_name = self.class_name_detail + object_id
                    print(f"IN CHECK [{data['author']}] object_id: {object_id} || class_name: {class_name} || video_id: {data['video_id']}")
                    if object_id is None:
                        break
                    else:
                        # 있으면 update 없으면 post
                        if self.task_baas.exist_key_date(class_name=class_name, keyName="video_id", data=data['video_id']):
                            self.task_baas.basicPut(class_name=class_name, object_id=object_id, data=data)
                            list_data.append(data)
                        else:
                            self.task_baas.basicPost(class_name=class_name, data=data)
                            list_data.append(data)

                except Exception as e:
                    print(e)
                    print(f"Exception object_id : {object_id} || class_name : {class_name} , {e}")
                    continue 
                finally:
                    print(f"[cron_youtube_redata_fn finnally] channel_id: {channel_id}")
                    self.update_rag_by_new(channel_id=channel_id)
        return list_data

    # 신규 동영상은 마스터 업데이트
    def youtube_new_data_and_master_update(self, watch_url:str, recent_data:dict ):
        video_id = self.task_common.get_audio_id(url=watch_url)
        channel_id = recent_data['channel_id']
        update_data = {
                    "title": recent_data['title'],
                    "thumbnail_url": recent_data['thumbnail_url'],
                    "watch_url": recent_data['watch_url'],
                    "publish_date": recent_data['publish_date'],
                    "video_length": recent_data['video_length'],
                    "description": recent_data['description'],
                    "video_id": video_id,
                }
        one  = self.task_baas.get_key_datas(class_name=self.class_name_master, keyName="channel_id", data=channel_id)
        # print(f"one: {one}")
        one_results = one['results']
        if len(one_results) > 0:
            print(f"video_id: {video_id} || recent_data: {recent_data}")
            object_id = one_results[0]['objectId']
            self.task_baas.basicPut(class_name=self.class_name_master, object_id=object_id, data=update_data)
        else:
            print(f"업데이트 대상이 없습니다. {recent_data['video_id']}")

    def cron_youtube_redata_fn(self, master):
        try:
            url = master['watch_url']
            data = self.get_yt_info(url)

            print(f"[cron_youtube_redata_fn] data: {master['author']}")
            self.save_yt_master(url=url, data=data)
            video_urls = [url]
            list_data = self.save_yt_detail(video_urls=video_urls, data=data, limit=10)

            if list_data is not None and len(list_data) > 0:

                recent_data = sorted(list_data, key=lambda x: x['publish_date'], reverse=True)[0]
                self.youtube_new_data_and_master_update(watch_url=url, recent_data=recent_data)
                
        except Exception as e:
            if "403" in str(e): 
                # 403 에러시 토큰 재생성
                self.task_common.regenerate_network_token(url=url)
                sleep(10)
            print(f"[cron_youtube_redata_fn] error: {e}")        


        # 오늘 날짜 유튜브 채널 정보 저장
    def target_youtube_daily(self):
        start_time = time.time()
        ## 삭제로직 먼저 실행
        self.task_baas.delete_all_class(class_name=self.class_name_daily)

        ## YoutubeMaster 조회 후 저장
        datas = self.get_youtube_master_info()
        

        for index, data in enumerate(datas):
            author = data['author']
            objectId = data['objectId']
            updateObjectId = data['objectId']
            publish_date = data['publish_date']
            check = self.check_range_date(publish_date=publish_date)
            self.cron_youtube_daily_recent_week(check, objectId, author, index, publish_date, updateObjectId)

        end_time = time.time()
        print(f"[target_youtube_daily] time: {end_time - start_time}")

    def cpu_count(self):
        return multiprocessing.cpu_count()

    # 전부삭제하고 새로데이터 추가
    def target_youtube_remove_and_add(self):
        start_time = time.time()

        master_data = self.get_youtube_master_info(order_key="-publish_date")
      
        for master in master_data:
            author = master['author']
            print(f"[target_youtube_remove_and_add] master: {author}")
            self.cron_youtube_redata_fn(master)

        end_time = time.time()
        print(f"[target_youtube_remove_and_add] time: {end_time - start_time}")

    # 전부삭제하고 새로데이터 추가
    def target_youtube_remove_and_add_single(self, channel_id:None):
        start_time = time.time()

        master_data = self.get_youtube_master_info(order_key="-publish_date")
      
        print(f"[target_youtube_remove_and_add] master_data length: {len(master_data)}")
        if channel_id is not None:
            master_data = [master for master in master_data if master['objectId'] == channel_id]

        for master in master_data:
            author = master['author']
            print(f"[target_youtube_remove_and_add] master: {author}")
            self.cron_youtube_redata_fn(master)

        end_time = time.time()
        print(f"[target_youtube_remove_and_add] time: {end_time - start_time}")    


    # 1주일 기준 데일리 크론 작동 함수
    def cron_daily_recent_videos(self):
        self.target_youtube_daily()


    # 오늘 날짜 유튜브 채널 정보 저장
    def cron_youtube_reget_datas(self):
        self.target_youtube_remove_and_add()

    def cron_youtube_reget_datas_single(self, channel_id:None):
        self.target_youtube_remove_and_add_single(channel_id=channel_id)    



    def get_master_list(self):
        clss_name_master = "YoutubeMaster"
        res = self.task_baas.basicGet(class_name=clss_name_master)
        return res

    def background_daglo_master_stt_cron(self):
        res = self.get_master_list()
        results = res.json()['results']
 
        for res_master in results:
            url = res_master['watch_url']
            print(f"[STT] url: {url}")
            self.daglo_agent.run(url)
        

    def background_daglo_master_rag_cron(self):
        res = self.get_master_list()
        results = res.json()['results']
        thread_list = []
        for res_master in results:
            url = res_master['watch_url']
            print(f"[RAG] url: {url}")
            self.daglo_agent.rag_run(url)


    def background_add_youtube_channel(self, video_url:str):
        data = self.get_yt_info(video_url)
        self.save_yt_master(url=video_url, data=data)
        video_urls = [video_url]
        self.save_yt_detail(video_urls=video_urls, data=data, limit=10)
        return data      


    def whisper_action(self, url:str):
        self.agent_whisper.rag_run(url=url)
        return True        



    def cron_whisper_cron(self):
        print(f"cron_whisper_cron start")
        res = self.get_master_list()
        results = res.json()['results']
        for index, res_master in enumerate(results):
            url = res_master['watch_url']
            author = res_master['author']
            self.whisper_action(url)


    # 업데이트 안된 영상 처리
    def cron_whisper_not_updated(self):
        # master 에서 대상채널 조회
        master_res = self.get_master_list()
        master_results = master_res.json()['results']
        for master_result in master_results:
            # author = master_result['author']
            channel_id = master_result['channel_id']
            self.update_rag_by_new(channel_id=channel_id)
    

                


            
          




              
