import ngrok
import os
import json
from ..tasks.daglo_task import DagloTask
from ..tasks.youtube_task import YoutubeTask
from ..tasks.ha_common_task import HaCommonTask
from ..tasks.file_task import FileTask
from ..ha_constant import HaConstant
from ..model.youtube_info import YoutubeInfo
from ..tasks.baas_clsname_task import BassClsNameTask
from time import sleep
from datetime import datetime
from ..agents.ha_youtube_agent import HaYoutubeAgent
from ..agents.naver_agent import NaverAgent
from ..tasks.rag_task import RAGTask

ngrok.set_auth_token(os.getenv("NGROK_AUTH_TOKEN"))



class DagloAgent:
    def __init__(self):
        self.agent_youtube = HaYoutubeAgent()
        self.agent_naver = NaverAgent()
        self.task_daglo = DagloTask()
        self.task_youtube = YoutubeTask()
        self.task_common = HaCommonTask()
        self.task_baas = BassClsNameTask()
        self.task_file = FileTask()
        self.task_rag = RAGTask()
        self.embed_dir = HaConstant.embed_dir
        self.rid = None
        self.clss_name_daglo = "DagloWork"
        self.clss_name_master = "YoutubeMaster"
        self.clss_name_detail = "YoutubeDetail_"


    ## 1. 유튜브 다운로드 해서 mp3 파일이 생성이 되면
    ## 2. 다글로 서버에 전송
    ## 3. 다글로 rid로 체킹 

    def run(self, url:str):
        audio_id = self.task_common.get_audio_id(url)
        yt_info = self.task_common.get_youtube_info(url)
        youtube_info = YoutubeInfo()
        data = youtube_info.change_model(yt=yt_info)

        exist_mp3 = self.task_file.exist_audio_file(audio_id=audio_id)
        exist_txt = os.path.exists(f"{self.embed_dir}/{audio_id}.txt")


        if not exist_mp3:            
            self.step1_youtube_to_mp3(url)
            print("step1_youtube_to_mp3")
               
        while self.rid is None and not exist_txt:
            self.step2_ngrok_start_and_send(audio_id=audio_id, youtube_info=data)
            print(f"step2_ngrok_start_and_send rid: {self.rid}")
            sleep(10)
            if self.rid is not None:
                break
            else:
                sleep(10)
      
        while not exist_txt and self.rid is not None:
            res = self.step3_rid_check(rid=self.rid, audio_id=audio_id, youtube_info=data)
            print(f"step3_rid_check: {res}")
            
            sleep(10)
            if res and self.step4_daglo_check_and_write_txt(res, audio_id=audio_id, rid=self.rid, youtube_info=data):
                print("step4_daglo_check_and_write_txt")
                # 폴더내 mp3 파일 삭제
                self.task_file.delete_audio_file(audio_id=audio_id)
                break
            else:
                sleep(10)
        if exist_mp3:
            self.task_file.delete_audio_file(audio_id=audio_id)        
        print(f"run end")

    # 영상 처리 대기건 처리용
    def action_wait_daglo_work(self, url):
        audio_id = self.task_common.get_audio_id(url)
        res = self.task_baas.get_key_datas(class_name=self.clss_name_daglo, keyName="audio_id", data=audio_id)
        step = res['results'][0]['step']
        rid = res['results'][0]['rid']
        if step == 'RECIEVED_RID':
            self.get_daglo_res_and_after(url=url, rid=rid, audio_id=audio_id)

    # rid 받았으나 이후 프로세스 진행이 안된경우에 돌리는 함수
    def get_daglo_res_and_after(self, url:str, rid:str, audio_id:str):
        yt_info = self.task_common.get_youtube_info(url)
        youtube_info = YoutubeInfo()
        data = youtube_info.change_model(yt=yt_info)
        exist_txt = self.task_file.exist_text_file(audio_id=audio_id)
        print(f"get_daglo_res_and_after start exist_txt: {exist_txt}")
        while not exist_txt and rid is not None:
            res = self.step3_rid_check(rid=rid, audio_id=audio_id, youtube_info=data)
            print(f"step3_rid_check: {res}")
            
            sleep(30)
            if res and self.step4_daglo_check_and_write_txt(res, audio_id=audio_id, rid=rid, youtube_info=data):
                print("step4_daglo_check_and_write_txt")
                # 폴더내 mp3 파일 삭제
                self.task_file.delete_audio_file(audio_id=audio_id)
                break
            else:
                sleep(10)
        print(f"get_daglo_res_and_after end")

    def step1_youtube_to_mp3(self, url:str):
        self.task_youtube.open_and_get_audio(url)

    def step2_ngrok_start_and_send(self, audio_id:str, youtube_info:dict):
        
        channel_id = youtube_info["channel_id"]
        title = youtube_info["title"]
        all_listener = ngrok.get_listeners()
        try:
            # print(f"all_listener: {all_listener}")
            if len(all_listener) > 0:
                if all_listener[0]:
                    server_url = all_listener[0].url()
                    print(f"server_url: {server_url}")

                    ## 3. daglo 서버로 전송
                    audio_url = f"{server_url}/files/{audio_id}.mp3"
                    print(f"audio_url: {audio_url}")
                    exist_mp3 = self.task_file.exist_audio_file(audio_id=audio_id)
                    exist_txt = os.path.exists(f"{self.embed_dir}/{audio_id}.txt")
                    if exist_mp3 and self.rid is None:
                        if not exist_txt:
                            send_res = self.task_daglo.send_daglo_unit(audio_file_path=audio_url)
                            self.rid = send_res["rid"]
                            cron_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                            self.task_baas.basicPost(class_name=self.clss_name_daglo, 
                                                 data={"rid":self.rid, "audio_id":audio_id, "channel_id":channel_id, 
                                                       "title":title, "step":"RECIEVED_RID", "cron_date":cron_date})
                    else:
                        if self.rid is not None:
                            send_res = self.task_daglo.get_daglo_unit(rid=self.rid)
            
                    return self.rid
                else:
                    for i, listener in enumerate(all_listener):
                        if i != 0:
                            listener.kill()
    
            else:
                print("else")
                listener = ngrok.forward(8000, authtoken_from_env=True)
                listener.url()
                print(f"else: {listener.url()}")
        except Exception as e:
            for listener in all_listener:
                url =  listener.url()
                print(f"exception url: {url}")
                ngrok.disconnect(url)
            print(f"error: {e}")    
            cron_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            self.task_baas.basicPost(class_name=self.clss_name_daglo, 
                                                 data={"rid": '', "audio_id":audio_id, "channel_id":channel_id, 
                                                       "title":title, "step":"ERROR", "msg":str(e), "cron_date":cron_date})
    

    def step3_rid_check(self, rid:str, audio_id:str, youtube_info:dict):
        res = self.task_daglo.get_daglo_unit(rid=rid)
        channel_id = youtube_info["channel_id"]
        title = youtube_info["title"]
        isError ='error' in res
        if isError:
            return False
        else:
            status = res['status']
            progress = res['progress']
            cron_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            self.task_baas.basicPost(class_name=self.clss_name_daglo, 
                                 data={"rid": '', "audio_id":audio_id, "channel_id":channel_id, 
                                                       "title":title, "step":status, "msg": f"{progress}%", "cron_date":cron_date})
        return res

    def step4_daglo_check_and_write_txt(self, res:dict, audio_id:str, rid:str, youtube_info:dict):
        
        status = res['status']
        channel_id = youtube_info["channel_id"]
        title = youtube_info["title"]
        if status == "transcribed":
            sttResult = res['sttResults'][0]['transcript']
            if not os.path.exists(f"{self.embed_dir}"):
                os.makedirs(f"{self.embed_dir}")
            with open(f"{self.embed_dir}/{audio_id}.txt", "w", encoding="utf-8") as f:
                f.write(sttResult)
            cron_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            self.task_baas.basicPost(class_name=self.clss_name_daglo, 
                                     data={"rid":rid, "step":"COMPLETED", "audio_id":audio_id, "channel_id":channel_id, 
                                                       "title":title, "msg": f"transcribed", "cron_date":cron_date})
            return True
        else:
            return False
        
    def get_yt_info(self, url):
        yt_info = self.agent_youtube.get_yt(url)
        youtube_info = YoutubeInfo()
        data = youtube_info.change_model(yt=yt_info)
        return data      
    
    def save_or_update_report(self, class_name=None, video_id=None, data=None):
        try:
            video_id =  self.task_baas.get_object_id(class_name=class_name, keyName="video_id", data=video_id)
            print(f"video_id: {video_id}")
            if video_id:
                # 업데이트
                res =self.task_baas.basicPut(class_name=class_name, object_id=video_id, data=data)
                print(f"UPDATE res: {class_name} {video_id} {res.status_code} {res.content}")
            else:
                # 신규 저장
                res = self.task_baas.basicPost(class_name=class_name, data=data)
                print(f"INSERT res: {class_name} {res.status_code}")
        except Exception as e:
            print(f"Error: {e}")

    def get_class_name(self, channel_id):
        object_id = self.task_baas.get_object_id(class_name=self.clss_name_master,keyName="channel_id", data=channel_id)

        class_name = self.clss_name_detail+object_id
        return class_name 

    def rag_run(self, url:str):
        data = self.get_yt_info(url)
        channel_id = data['channel_id']
        object_id = self.get_class_name(channel_id)
        texts = ["내용을 요약해서 정리해줘", 
                "내용안에서 언급된 책이 있으면 책 제목을 정리해서 알려줘",
                "내용안에서 언급된 사람이 있으면 이름을 정리하서 알려줘",
                "내용안에서 발췌해서, 유튜브 조회수 100만 가능한 제목으로 10개 만들어줘", 
                "내용안에서 발췌해서, 유튜브 조회수 100만 가능한 해시태그 10개 만들어줘", 
                "내용을 4지선다형 객관식 문제로 10개 만들어줘"]
        data = {}
        for index, text in enumerate(texts):
            audio_id = self.task_common.get_audio_id(url)
            rag_chain = self.task_rag.get_chain(url= url, file_path=f"{self.embed_dir}/{audio_id}.txt", index=index)
            res = rag_chain.invoke({"input": text, "index": index})
            answer = res["answer"].replace("```json", "").replace("```", "")
            if index == 0:
                data["summary"] = answer
            elif index == 1:
                # print(f"books: {answer}")

                # {"result":["미치게 만드는 브랜드","홈페이지 마케팅 플랜","모든 비즈니스는 브랜딩이다","원페이지 마케팅 플랜"]}
                try:
                    if type(answer) == str:
                        books = json.loads(answer)['result']
                        book_infos =[]
                        for book in books:
                            print(f"book: {book}")
                            book_info = self.agent_naver.book_search(book)
                            book_infos.append(book_info)

                        data["book_info"] = book_infos
                    else:
                     data["book_info"] = []
                except Exception as e:
                    print(f"Error: {e}")
                    data["book_info"] = []
            elif index == 2:
                data["persons"] = answer
            elif index == 3:
                data["titles"] = answer
            elif index == 4:
                data["hashtags"] = answer
            elif index == 5:
                data["questions"] = answer

        self.save_or_update_report(class_name=object_id, video_id=audio_id, data=data)
        return data
    
    