
from pydantic import BaseModel, Field
from pytubefix import YouTube
from datetime import datetime
from langchain.schema import Document

class YoutubeInfo(BaseModel):
    title: str = Field(default="")
    channel_id: str = Field(default="")
    video_id: str = Field(default="")
    author: str = Field(default="")
    description: str = Field(default="")
    thumbnail_url: str = Field(default="")
    channel_url: str = Field(default="")
    watch_url: str = Field(default="")
    publish_date: str = Field(default="")
    video_length: float = Field(default=0)
    views: int = Field(default=0)
    keywords: list[str] = Field(default=[])
    channel_thumbnail: str = Field(default="")
    channel_description: str = Field(default="")
    transcribe_text: list[Document] = Field(default=[])
    summary_text: list[Document] = Field(default=[])
    embedding_text: list[Document] = Field(default=[])

    def change_model(self, yt:YouTube)->dict:
        self.title = yt.title
        self.channel_id = yt.channel_id
        self.video_id = yt.video_id
        self.author = yt.author
        self.description = yt.description
        self.thumbnail_url = yt.thumbnail_url
        self.channel_url = yt.channel_url
        self.watch_url = yt.watch_url
        self.publish_date = yt.publish_date.strftime("%Y-%m-%d %H:%M:%S")
        self.video_length = round(yt.length / 60, 2)
        self.views = yt.views
        self.keywords = yt.keywords
        self.channel_thumbnail = yt.channel_thumbnail
        self.channel_description = yt.channel_description
        return self.model_dump()

    def to_dict(self):
        return self.model_dump()
