

from fastapi.security import APIKeyHeader
from fastapi import FastAPI, Depends, HTTPException, status, APIRouter, Header, BackgroundTasks

from ..agents.ha_youtube_agent import HaYoutubeAgent
from ..agents.cron_agent import CronAgent
from ..agents.daglo_agent import DagloAgent
from ..tasks.baas_clsname_task import BassClsNameTask
from ..ha_constant import HaConstant
from pydantic import BaseModel
from ..agents.rag_agent import RAGAgent
from ..model.rag_basic_model import RAGBasicModel
class MemberRouter:
    def __init__(self):
        pass

    async def api_token(token: str = Depends(APIKeyHeader(name="HA-X-API-KEY"))):
        if token is None:
            raise HTTPException(status_code=status.HTTP_403_FORBIDDEN)
        return token

    member = APIRouter(prefix="/member",dependencies=[Depends(api_token)])



    @member.post("/youtube/master")
    def read_youtube(): #, openai_key: str = Depends(api_token)):
        youtube_agent = HaYoutubeAgent()
        data = youtube_agent.get_youtube_master_info()
        return data

    @member.post("/youtube/detail")
    def read_youtube_detail(id:str):
        print(f"youtube_detail: {id}")
        youtube_agent = HaYoutubeAgent()
        res = youtube_agent.get_youtube_detail_recent(object_id=id)
        return res


    @member.post("/youtube/daily")
    def read_youtube_daily():
        youtube_agent = HaYoutubeAgent()
        res = youtube_agent.get_youtube_daily()
        return res

    @member.post("/youtube/ordered/master")
    def read_youtube_ordered_master():
        youtube_agent = HaYoutubeAgent()
        res = youtube_agent.get_youtube_master_order_publish_date()
        return res
    

    @member.post("/youtube/delete/channel")
    def read_youtube_delete_master(object_id:str):
        youtube_agent = HaYoutubeAgent()
        res = youtube_agent.delete_youtube_master_and_detail(object_id=object_id)
        if res.status_code == 200:
            return {"status": "SUCCESS"}
        else:
            return {"status": "FAIL"}

    class YoutubeDeleteDetail(BaseModel):
        class_id: str
        movie_id: str

    @member.post("/youtube/delete/detail")
    def read_youtube_delete_detail(data:YoutubeDeleteDetail):
        youtube_agent = HaYoutubeAgent()
        res = youtube_agent.delete_youtube_detail(class_id=data.class_id, movie_id=data.movie_id)
        if res.status_code == 200:
            return {"status": "SUCCESS"}
        else:
            return {"status": "FAIL"}

    @member.post("youtube/channel/newbi", description="신규채널 추가")
    def read_youtube_channel_add(video_url:str):
        cron_agent = CronAgent()
        return cron_agent.background_add_youtube_channel(video_url=video_url)

    @member.post("/youtube/channel/react/one", description="단일 채널 크론 (신규동영상 추가 및 RAG 처리)")
    def read_youtube_channel_add_cron(id:str):
        cron_agent = CronAgent()
        cron_agent.cron_youtube_reget_datas_single(channel_id=id)
        return {"status": "실행되었습니다. 결과는 콘솔 참고하세요."}

    @member.post("/youtube/channel/rag", description="해당 영상 RAG")
    def rag_youtube_url(data:RAGBasicModel):
        agent = RAGAgent()
        res = agent.get_basic_chain(data=data)
        return res


    @member.post("/daglo/single/run", description="단일 영상 처리")    
    def read_daglo_single_run(url:str):
        daglo_agent = DagloAgent()
        res = daglo_agent.run(url)
        return res


    @member.post("/daglo/single/after", description="단일 영상 처리 후 처리")    
    def read_daglo_single_after(url:str):
        daglo_agent = DagloAgent()
        res = daglo_agent.get_daglo_res_and_after(url=url)
        return res


    
    


    def background_daglo_master_stt_cron():
        cron_agent = CronAgent()
        cron_agent.background_daglo_master_stt_cron()

    @member.post("daglo/master/stt/cron", description="마스터 영상 스트리밍 처리")
    def read_daglo_master_stt_cron(background_tasks:BackgroundTasks,background_daglo_master_stt_cron= Depends(background_daglo_master_stt_cron)):
        background_tasks.add_task(background_daglo_master_stt_cron)
        return {"status": "실행되었습니다. 결과는 콘솔 참고하세요."}


    def background_daglo_master_rag_cron():
        cron_agent = CronAgent()
        cron_agent.background_daglo_master_rag_cron()

    @member.post("daglo/master/rag/cron", description="마스터 영상 RAG 처리")
    def read_daglo_master_rag_cron(background_tasks:BackgroundTasks,background_daglo_master_rag_cron= Depends(background_daglo_master_rag_cron)):
        background_tasks.add_task(background_daglo_master_rag_cron)
        return {"status": "실행되었습니다. 결과는 콘솔 참고하세요."}
        

    ## 아래 next-auth 관련 코드 추가
    @member.get("/auth/providers")
    def auth_providers():
        return {"status": ["github","google","kakao"]}

    @member.get("/auth/session")
    def auth_session():
        return {"status": "ok"}

    @member.post("/auth/_log")
    def auth_log():
        return {"status": "ok"}

    @member.get("/auth/signin")
    def auth_signin(callbackUrl:str):
        return {"status": "ok"}

    @member.post("/auth/signout")
    def auth_signout():
        return {"status": "fail"}
    

    def get_router(self):
        return self.member


