from __future__ import print_function
import time
import brevo_python
from brevo_python.rest import ApiException
from pprint import pprint
from dotenv import load_dotenv
import os
load_dotenv()
import requests

from ..model.brevo_model import BrevoMailBody, BrevoMailType



class BrevoTask:
    def __init__(self):
        self.instance = self.get_instance()

    def get_header(self):
        request_header = {
            "Content-Type": "application/json",
            "api-key": os.getenv('BREVO_API_KEY'),
            "accept": "application/json"
        }
        return request_header

    def base_mail_send(self, data: BrevoMailBody):
        url = f"{os.getenv('BREVO_API_URL')}/v3/smtp/email"
        headers = self.get_header()
        response = requests.post(url, headers=headers, json=data.model_dump())
        return response


    def get_instance(self):
        # Configure API key authorization: api-key
        # print(f"BREVO_API_KEY: {os.getenv('BREVO_API_KEY')}")
        configuration = brevo_python.Configuration()
        configuration.api_key['api-key'] = os.getenv('BREVO_API_KEY')

        # create an instance of the API class
        try:
            api_instance = brevo_python.AccountApi(brevo_python.ApiClient(configuration))
            return api_instance
        except Exception as e:
            print(f"Error: {e}")
            return None
        



    def get_account(self):
        return self.instance.get_account()

    def get_contacts(self):
        return self.instance.get_contacts()
    
    def get_campaigns(self):
        return self.instance.get_campaigns()
    
    def get_campaign_stats(self):
        return self.instance.get_campaign_stats()
    
    def get_campaign_reports(self):
        return self.instance.get_campaign_reports()
    
    def get_campaign_report_by_id(self, campaign_id):
        return self.instance.get_campaign_report_by_id(campaign_id)
    
    def get_campaign_report_by_name(self, campaign_name):
        return self.instance.get_campaign_report_by_name(campaign_name)
    
