from langchain_community.document_loaders import AsyncChromiumLoader
from langchain_community.document_transformers import BeautifulSoupTransformer
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain_openai import ChatOpenAI
from pydantic import BaseModel, Field
from langchain.output_parsers import PydanticOutputParser
from langchain.prompts import ChatPromptTemplate
import json

class SermonTable(BaseModel):
    no: int = None
    title: str = None
    link: str = None
    word: str = None
    speaker: str = None
    date: str = None
    view: int = None

class BoardList(BaseModel):
    result : list[SermonTable] = None

class WebScripeTask():
    def __init__(self, url: str):
        self.url = url

    def get_html_bs4(self, selector):    
        loader = AsyncChromiumLoader([self.url])
        docs = loader.load()
        bs_transformer = BeautifulSoupTransformer()
        docs_transformer = bs_transformer.transform_documents(docs, tags_to_extract=[selector])
        
        content = docs_transformer[0].page_content  
        return content


    def run(self) :
        llm = ChatOpenAI()
        
        prompt = ChatPromptTemplate.from_messages([
            ("system", """
             당신은 주어진 형태를 참고하여 내용을 파싱하는 전문가이다.
             주어진 형태를 참고하여 내용을 파싱하세요.

             형태는 다음과 같습니다.
             no  title             link                                        word      speaker   date      view
             668 너희를 자유롭게 하리라 (board_view.html?code=1241&num=1179&page=1) 요한복음 8:31b3 박행신 목사 2024-12-01 84 
             667 너의 이름이 무엇이냐? (board_view.html?code=1241&num=1178&page=1)  창세기 32:2428 박행신 목사 2024-11-24 52

             최종결과물은  아래 Format에 맞는 json 형태로 출력하세요.


             Format : 
              {format}

 """),
            ("user", "{content}")
        ])

        
        bs_content = self.get_html_bs4("table")
        # text_splitter = RecursiveCharacterTextSplitter.from_tiktoken_encoder(chunk_size=1000, chunk_overlap=200)
        # splits = text_splitter.split_documents(content)
        parser = PydanticOutputParser(pydantic_object=BoardList)        
        prompt = prompt.partial(format=parser.get_format_instructions())

        # print(content)

        chain = prompt | llm 
        response = chain.invoke({"content": bs_content})
        json_content = json.loads(response)["response"]

        pycontent = parser.parse(json_content)
   
        # print(pycontent)
        return pycontent
