import os
import ssl

from pytubefix import YouTube
from urllib.parse import urlparse

from langchain_community.document_loaders.blob_loaders.youtube_audio import (
    YoutubeAudioLoader,
)
from langchain_community.document_loaders.generic import GenericLoader
from langchain_community.document_loaders.parsers import (
    OpenAIWhisperParser,
)

from ..ha_constant import HaConstant
from ..tasks.ha_common_task import HaCommonTask


ssl._create_default_https_context = ssl._create_unverified_context

class YoutubeTask:
    def __init__(self, ):
        self.common_task = HaCommonTask()


    def download_file(self, videourl, path=HaConstant.video_dir):
        yt = self.common_task.getYoutubeInfo(videourl)
        audio_id = self.common_task.get_audio_id(videourl)
        yt = yt.streams.get_highest_resolution()
        if not os.path.exists(path):
            os.makedirs(path)
        return yt.download(output_path=path, filename=f"{audio_id}.mp3")


        
    def open_and_get_audio(self, youtube_url:str) -> tuple[str, str, str]:
        # open video
        video_path = HaConstant.audio_dir
        videostr = self.download_file(youtube_url, path=video_path)

        video =  open(videostr, "rb")
        if video:
            video_content = video.read()
            video_path = f"{video.name}"
            with open(video_path, "wb") as f:
                f.write(video_content)

            return video_path  
        


    ###
    ### 순수 langchain 사용 stt처리 
    def stt_by_langchain(self, url:str):
        
        audio_id = self.common_task.get_audio_id(url)
             
        urls = [url]
        audio_dir = HaConstant.audio_dir
        text_dir = HaConstant.text_dir
        if self.common_task.exist_stt_file(audio_id):
            return f"{text_dir}/{audio_id}.txt"
        loader = GenericLoader(
            blob_loader=YoutubeAudioLoader(urls, audio_dir), 
            blob_parser=OpenAIWhisperParser()
        )
        docs = loader.load()

        

        file_path = f"{text_dir}/{audio_id}.txt"
        with open(file_path, "wb") as f:
            for doc in docs:
                f.write(doc.page_content.encode("utf-8"))
            f.close()

        docs_meta = docs[0].metadata
        print(docs_meta)
        title_source = docs_meta["source"]
        title = title_source.replace(".m4a","").replace("{audio_dir}/","")
        result = {
            "title": title,
            "id": audio_id,
            "txt_path": f"{text_dir}/{audio_id}.txt",
            "meta": docs_meta
        }    

        return result
